from __future__ import annotations
from typing import List

class TaalcToken():
    name: str
    aliases: List[str]

    def __init__(self, name, aliases = None):
        self.name = name
        if aliases:
            self.aliases = aliases

    @classmethod
    def get_by_alias(cls, alias: str) -> TaalcToken:
        pass


from __future__ import annotations
from .taalc_token import TaalcToken
from epure import epure
from typing import List

@epure()
class Currency(TaalcToken):
    _currencies = None
    aliases: List[str]

    @classmethod
    def currencies(cls) -> List[Currency]:
        if not cls._currencies:
            cls._currencies = Currency.resource.read()
        return cls._currencies
   

    @classmethod
    def get_by_alias(cls, alias: str) -> TaalcToken:
        res = None

        for cr in cls.currencies():
            if cr.aliases and alias in cr.aliases:
                res = cr
                break
        if res == None:
            raise ValueError(f'there is no currency with {alias} alias')
        
        return res
    
    def __str__(self):
        return self.aliases[0]


from aiogram import Bot, Dispatcher, types, Router
from ..tg_environment.t_user import TUser
from ..decorators import msg_handler
from ..finance.currency import Currency

# def gift(message: types.Message, user: User, msg_text: str):
#     pass

@msg_handler(r'(?i).*марат.*')
async def process_message(message: types.Message, user: TUser, match):
    msg_text = message.text.lower()
    if message.reply_to_message and \
        ('марат передай' in msg_text or 'марат, передай' in msg_text):

        msg_split = msg_text.split()
        cur_alias = msg_split[-1]
        currency = Currency.get_by_alias(cur_alias)
        amount = float(msg_split[-2])
        wallet_amount = user.wallet.amount(currency)
        if amount <= 0 or wallet_amount <= 0:
            return await message.reply('А нахуй сходить не хочешь?')
            
        
        if amount > wallet_amount:
            res = f"У тебя нет столько {currency.aliases[1]}, кого ты пытаешься наебать? "+\
                f"У тебя всего лишь {wallet_amount} грамм, иди поработай жопой, нищук."
            return await message.reply(res)
            

        # to_user = User.resource.read(telegram_id = message.reply_to_message.from_user.id)[0]
        to_user = TUser.user_by_msg(message.reply_to_message)
        transaction = user.send_currency(to_user, currency, amount)

        return await message.reply_to_message.reply(f"{to_user}, {user} передал тебе {currency.aliases[1]}, "+ \
                            f"{amount} грамм, запрвляй баян")
    elif message.reply_to_message and \
        ('марат, петух' in msg_text or 'марат петух' in msg_text):
        res = "А твоя мамка дешевая подзаборная шлюха, и что? " +\
            "Ну давай посмотрим сколько этот петушок заработал своим очком: \n"
        
        checked_user = TUser.user_by_msg(message.reply_to_message)            
        total = 0
        for cr in Currency.currencies():
            amt = checked_user.wallet.amount(cr)
            total += amt
            res += f'{cr.aliases[0]}: {amt} грамм \n'
        if total <= 300:
            res += f'Петушок {checked_user} похож на нищука, скоро пойдёт нахуй отсюда!'
        else:
            res += f'Похоже петушок {checked_user} неплохо работает жопой!'

        return await message.reply(res)

    elif msg_text in ('марат, я петух', 'марат я петух'): 
        pass
    elif 'пиво' in message.text.lower():
        return await message.reply(f"где сходка?")
    else:
        return await message.reply(f"{message.from_user.first_name} - шлюха")


from epure import epure
from .taalc_transaction import TaalcTransaction
from .taalc_nft import TaalcNft
from .taalc_nft_token import TaalcNftToken

@epure()
class NftTransaction(TaalcTransaction):
    taalc_nft: TaalcNft
    taalc_nft_token: TaalcNftToken


from .tokens_bag import TokensBag
from ..tg_environment.t_user import TUser
from .currency import Currency
from .currency_transaction import CurrencyTransaction

class Wallet(TokensBag):

    user: TUser
    # currency: Currency

    def __init__(self, user: TUser):
        self.user = user
        # self.currency = currency

    def amount(self, currency: Currency) -> float:
        received = CurrencyTransaction.resource.read(sent_to=self.user.data_id, currency=currency.data_id)
        total_input = sum(tr.amount for tr in received)

        spent = CurrencyTransaction.resource.read(sent_from=self.user.data_id, currency=currency.data_id)
        total_output = sum(tr.amount for tr in spent)

        res = total_input - total_output
        return res


from typing import TYPE_CHECKING
from ..finance.taalc_nft import TaalcNft
from .t_user import TUser
from .t_chat import TChat
from .. import bidding
# if TYPE_CHECKING:
#     from ..bidding.t_offer import TOffer
from epure import epure
from aiogram import types

@epure()
class TMessage():
    owner: TUser
    creator: TUser
    # if TYPE_CHECKING:
    taalc_offer: 'bidding.t_offer.TOffer' = None
    taalc_nft: TaalcNft = None
    taalc_chat: TChat
    tg_chat_id: int
    tg_message_id: int

    def __init__(self, message: types.Message):
        
        self.creator = TUser.user_by_tg_user(message.from_user)
        self.owner = TUser.user_by_tg_user(message.from_user)
        self.tg_chat_id=message.chat.id
        self.tg_message_id=message.message_id

        taalc_chat = TChat.resource.read(tg_chat_id=self.tg_chat_id)
        if taalc_chat:
            taalc_chat = taalc_chat[0]
        else:
            taalc_chat = TChat()
            taalc_chat.tg_chat_id = message.chat.id
            taalc_chat.shifted_id = message.chat.shifted_id
        self.taalc_chat = taalc_chat

    @classmethod
    def get_t_message(cls, message: types.Message):
        res = cls.resource.read(tg_chat_id=message.chat.id, \
                                tg_message_id=message.message_id)
        if not res:
            res = cls(message)
            res.save()
        else:
            res = res[0]
        
        return res
    
    def get_url(self):
        res = f"https://t.me/c/{self.taalc_chat.shifted_id}/{self.tg_message_id}"
        return res
    

        


from aiogram.types import Message

class BotResponse:
    response: Message = None
    orig: Message = None
    is_responded: bool = False

    def __init__(self, request):
        self.orig = request


from pytest import fixture, mark
from ..teest.tester import Tester
from epure.files import IniFile
from .. import __main__ as main
from ..taalc_bot import TaalcBot

@fixture
async def user1():
    config = IniFile("./pyt/pyconfig.ini")
    tested_bot = main.get_bot(config)
    # tested_bot = None
    tested_bot.start()
    res = Tester(config.test_user1_token, tested_bot, config.test_chat_id)
    return res

@mark.asyncio
async def test_process_message(user1):
    user1 = await user1
    
    res = await user1.msg('hellow world')
    assert res.is_responded == False

    res = await user1.msg('марат')
    assert res.response and 'шлюха' in res.response.text.lower()

    res = await user1.msg('привет, марат')
    assert res.response and 'шлюха' in res.response.text.lower()
    user1.tested_bot.stop()


# from __future__ import annotations
# from typing import Optional, TYPE_CHECKING
from epure import epure
from .currency import Currency
from ..tg_environment.t_member import TMember
# if TYPE_CHECKING:
from .taalc_transaction import TaalcTransaction
from datetime import datetime

@epure()
class CurrencyTransaction(TaalcTransaction):

    currency: Currency
    amount: float


    def __init__(self, sent_from, sent_to, currency, amount):

        self.sent_from = sent_from
        self.sent_to = sent_to
        self.currency = currency
        self.amount = amount
        self.transaction_time = datetime.now()